###########################################################################
#
#   Copyright (c) 2000,2001 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000,2001 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import OFS.Folder
import Products
import string
import bookmark
import product_base_item
import base_item

bookmark_folder_redirect = HTMLFile('bookmark_folder_redirect', globals())
manage_add_bookmark_folder_form = HTMLFile('bookmark_folder_add', globals())
manage_add_bookmark_folder_internal_form = HTMLFile('bookmark_folder_internal_add', globals())


def manage_add_bookmark_folder(self, id, title, description,
				install_default_html=None,
				install_default_theme=None,
				install_default_id_generator=None,
				REQUEST=None):
	"""Add an instance of the bookmark_folder class."""

	self._setObject(id, bookmark_folder(id, title, description,
				install_default_html=install_default_html,
				install_default_theme=install_default_theme,
				install_default_id_generator=install_default_id_generator))

	if REQUEST is not None:

		REQUEST.RESPONSE.redirect(self.absolute_url())

	else:

		return self[id]


class bookmark_folder(
	Products.ZCatalog.CatalogAwareness.CatalogAware,
	OFS.Folder.Folder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	product_base_item.product_base_item,
	base_item.base_item
	):
	"""Class for grouping bookmarks."""

	meta_type = 'BookmarkFolder'
	icon = 'misc_/BookmarkFolder/Folder_icon.gif'
	default_catalog = 'Catalog'
	product_name = 'BookmarkFolder'
	help_name = 'bookmark_folder.stx'

	meta_types = (
		{'name':'Bookmark',
			'action':'manage_add_bookmark_form'},
		{'name':'BookmarkFolder',
			'action':'manage_add_bookmark_folder_internal_form'}
		)

	manage_add_bookmark_folder_internal_form = manage_add_bookmark_folder_internal_form
	manage_add_bookmark_folder = manage_add_bookmark_folder

	manage_add_bookmark_form = bookmark.manage_add_bookmark_form
	manage_add_bookmark = bookmark.manage_add_bookmark

	_properties = (
		{'id':'title', 'type':'string'},
		{'id':'description', 'type':'text'},
		)

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('View BookmarkFolders',
		 ('', 'display', 'get_bookmark_folders',
			'get_bookmarks', 'get_unique_id',
			)),
		('Manage bookmarks',
			('manage_add_bookmark_form', 'manage_add_bookmark'))
		)


	index_html = HTMLFile('bookmark_folder_index', globals())
	display = HTMLFile('bookmark_folder_display', globals())


	def render_title(self):
		"""Returns the title, if the title is empty, returns 'No title'."""

		title = self.title

		if string.strip(title):

			return title

		else:

			return 'No title'


	def get_bookmark_folders(self):
		"""Returns a list of the bookmark folders contained."""

		return self.objectValues('BookmarkFolder')


	def get_bookmarks(self):
		"""Returns a list of the bookmarks contained."""

		return self.objectValues('Bookmark')


	def local_frame_navigation(self, *args, **kw):
		"""Returns frame navigation."""

		return apply(self.local_navigation, args, kw)


	def __init__(self, id, title='', description='',
			install_default_html=None,
			install_default_theme=None,
			install_default_id_generator=None):
		"""Initialize the class instance."""

		self.id = id
		self.title = title
		self.description = description

		product_base_item.utils.default_installation(self,
					install_default_html=install_default_html,
					install_default_theme=install_default_theme,
					install_default_id_generator=install_default_id_generator)

default__class_init__(bookmark_folder)

