###########################################################################
#
#   Copyright (c) 2001 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2001 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.0 $

import string
import time
import utils
import html_widgets
import DateTime

class dummy_widget:

	def __call__(self):

		return ''


class time_widget:
	"""Used for manipulating time values in HTML."""

	composite_type = 'time'

	def __init__(self, name='', hour=None, minute=None,
			seconds=None, composite=None):

		if seconds is not None:

			hour = seconds / (60*60)
			minute = (seconds % (60*60)) / 60

		self.name = name
		self.hour = hour
		self.minute = minute
		self.composite = composite


	def __call__(self, separator_html="&nbsp;"):
		"""Returns an HTML presentation of the widget."""

		hour_name = self.name + '_hour'
		minute_name = self.name + '_minute'

		if self.composite:

			composite_tag = dummy_widget()

		else:

			composite_tag = html_widgets.hidden(
				name='__composite_value;%s;%s' % 
				(self.name, self.composite_type))

		hour_widget = utils.make_range_select(hour_name, 0, 24,
				selected=self.hour, pad=1)
		minute_widget = utils.make_range_select(minute_name, 0, 60,
					step=5, selected=self.minute, pad=1)

		return composite_tag() + string.join(
			[hour_widget(), minute_widget()], separator_html)


class date_widget:
	"""Used for manipulating date values in HTML."""

	composite_type = 'date'

	def __init__(self, name='', year=None, month=None, day=None,
			year_offset=10, start_year=None, composite=None,
			datetime=None):

		if datetime is not None:

			year = datetime.year()
			month = datetime.month()
			day = datetime.day()

		self.name = name
		self.year = year
		self.month = month
		self.day = day
		self.year_offset = year_offset
		self.start_year = start_year or year
		self.composite = composite


	def __call__(self, separator_html="&nbsp;"):
		"""Returns an HTML presentation of the widget."""

		year_name = self.name + '_year'
		month_name = self.name + '_month'
		day_name = self.name + '_day'

		if self.composite:

			composite_tag = dummy_widget()

		else:

			composite_tag = html_widgets.hidden(
					name='__composite_value;%s;%s' % 
					(self.name, self.composite_type))

		year_widget = utils.make_range_select(
				year_name, self.start_year,
				self.start_year + self.year_offset,
				selected=self.year)
		month_widget = utils.make_range_select(
				month_name, 1, 13, selected=self.month, pad=1)
		day_widget = utils.make_range_select(day_name, 1, 32,
				selected=self.day, pad=1)

		return composite_tag() + string.join(
				[year_widget(), month_widget(),
				day_widget()], separator_html)


class datetime_widget:
	"""Used for manipulating datetime values in HTML."""

	composite_type = 'datetime'


	def __init__(self, name='', datetime=None):

		self.name = name
		self.datetime = datetime or DateTime.DateTime()


	def __call__(self, separator_html="&nbsp;"):
		"""Returns an HTML presentation of the widget."""

		year = self.datetime.year()
		month = self.datetime.month()
		day = self.datetime.day()
		hour = self.datetime.hour()
		minute = self.datetime.minute()

		composite_tag = html_widgets.hidden(
					name='__composite_value;%s;%s' % 
					(self.name, self.composite_type))

		date_widget_instance = date_widget(name=self.name ,
				year=year, month=month, day=day, composite=1)
		time_widget_instance = time_widget(name=self.name,
				hour=hour, minute=minute, composite=1)

		return composite_tag() + string.join(
			[date_widget_instance(), time_widget_instance()],
						separator_html)

def test():

	return (date_widget(name='test-date'),
		time_widget(name='test-time'),
		datetime_widget(name='test-datetime'))

if __name__ == '__main__':

	d, t, dt = test()
