###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.1 $

__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import Products
import personal_container
import personal_file
import product_base_item
import base_item

personal_folder_add_redirect = HTMLFile('personal_folder_add_redirect', globals())
manage_add_personal_folder_form = HTMLFile('personal_folder_add', globals())

def manage_add_personal_folder(self, id, title, description,
				install_default_html='',
				install_default_theme='',
				install_default_id_generator='',
				REQUEST=None):
	"""Add an instance of the PersonalFolder class."""

	self._setObject(id, personal_folder(id, title, description,
				install_default_html=install_default_html,
				install_default_theme=install_default_theme,
				install_default_id_generator=install_default_id_generator))

	if REQUEST is not None:

		REQUEST.RESPONSE.redirect(self.absolute_url())

	else:

		return self[id]


class personal_folder(
	Products.ZCatalog.CatalogAwareness.CatalogAware,
	Products.BTreeFolder.BTreeFolder.BTreeFolder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	product_base_item.product_base_item,
	base_item.base_item
	):
	"""Container for managing files and folders."""

	meta_type = 'PersonalFolder'
	product_name = 'PersonalFolder'
	icon = 'misc_/PersonalFolder/Folder_icon.gif'
	default_catalog = 'Catalog'
	help_name = 'personal_folder.stx'

	_properties = (
		{'id':'title', 'type':'string'},
		{'id':'description', 'type':'text'},
		)

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('View PersonalFolders',
		 ('',)),
		)

	meta_types = (
		{'name':'PersonalContainer', 'action':'manage_add_personal_container_form'},
		{'name':'PersonalFile', 'action':'manage_add_personal_file_form'},
			)

	manage_add_personal_container_form = personal_container.manage_add_personal_container_form
	manage_add_personal_container = personal_container.manage_add_personal_container

	manage_add_personal_file_form = personal_file.manage_add_personal_file_form
	manage_add_personal_file = personal_file.manage_add_personal_file

	index_html = HTMLFile('personal_folder_index', globals())
	display = HTMLFile('personal_folder_display', globals())

	def __init__(self, id, title='', description='',
			install_default_html='',
			install_default_theme='',
			install_default_id_generator=''):
		"""Initialize the class instance."""

		Products.BTreeFolder.BTreeFolder.BTreeFolder.__init__(self)

		self.id = id
		self.title = title
		self.description = description

		product_base_item.utils.default_installation(
			self,
			install_default_html=install_default_html,
			install_default_theme=install_default_theme,
			install_default_id_generator=install_default_id_generator)

		self.index_object()


	def get_personal_containers(self):
		"""Returns a list of personal containers."""

		return self.objectValues('PersonalContainer')


	def get_personal_files(self):
		"""Returns a list of personal files."""

		return self.objectValues('PersonalFile')


default__class_init__(personal_folder)