###########################################################################
#
#   Copyright (c) 2000,2001 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000,2001 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import OFS.Folder
import Products
import product_base_item
import base_item
import util

# We need these to handle ConflictErrors

import ZODB.POSException
import time

manage_add_workspace_form = HTMLFile('workspace_add', globals())


def manage_add_workspace(self, id, title, description,
				use_notifications=None,
				use_frames=None,
				install_default_html=None,
				install_default_theme=None,
				install_default_id_generator=None,
				REQUEST=None):
	"""Add an instance of the workspace class."""

	self._setObject(id, workspace(id, title, description,
				use_notifications=use_notifications,
				use_frames=use_frames,
				install_default_html=install_default_html,
				install_default_theme=install_default_theme,
				install_default_id_generator=install_default_id_generator))

	if REQUEST is not None:

		REQUEST.RESPONSE.redirect(self.absolute_url())


class workspace(
	OFS.Folder.Folder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	product_base_item.product_base_item,
	base_item.base_item
	):
	"""Used for grouping workspace-like (groupware) objects."""

	meta_type = 'Workspace'
	icon = 'misc_/Workspace/Folder_icon.gif'
	default_catalog = 'Catalog'
	product_name = 'Workspace'

	use_local_navigation = 0
	use_breadcrumbs = 0

	# FIXME

	table_border0 = 0

	_properties = (
		{'id':'title', 'type':'string'},
		{'id':'description', 'type':'text'},
		{'id':'use_notifications', 'type':'boolean',
			'boolean_widget':'use_notification_options'},
		{'id':'use_frames', 'type':'boolean',
			'boolean_widget':'use_frames_options'
		},
		)

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('View Workspaces',
		 ('', 'get_global_bokmark_folder',
			'get_gum', 'get_address_book',
			'get_address_book', 'get_note_folder',
			'get_note_folder', 'get_bookmark_folder',
			'get_calendar', 'get_personal_folder',
			'get_workspace_actions'),
			('Owner', 'Manager')),
		)


	use_notification_options = \
		{
			-1:'Yes',
			0:'No'
		}

	use_frames_options = \
		{
			-1:'Yes',
			0:'No'
		}


	index_html = HTMLFile('workspace_index', globals())
	index_html_alternate = HTMLFile('workspace_index_alternate', globals())
	local_frame = HTMLFile('workspace_local_frame', globals())
	about = HTMLFile('workspace_about', globals())
	preferences = HTMLFile('workspace_preferences', globals())
	frames = HTMLFile('workspace_frames', globals())
	navigation_frame = HTMLFile('workspace_navigation_frame', globals())
	left_frame = HTMLFile('workspace_left_frame', globals())
	right_frame = HTMLFile('workspace_right_frame', globals())
	main_frame = HTMLFile('workspace_main_frame', globals())
	notification_window = HTMLFile('workspace_notification_window', globals())
	notification_received = HTMLFile('workspace_notification_received', globals())


	def local_frame_navigation(self):
		"""Returns HTML for frame navigation."""

		navigation_html = """<select onchange="top.frames.main_frame.document.location.href='%s/'+this.options[this.selectedIndex].value;this.selectedIndex=0"><option value="%s/base_edit" disabled>Preferences</option>""" % \
			(self.REQUEST['URL1'], self.relative_url())

		for action in self.get_workspace_preferences_actions():

			navigation_html = navigation_html + """<option value="%(action)s">%(title)s</option>""" % action

		navigation_html = navigation_html + "</select><br>"
		navigation_html = navigation_html + """<select onchange="top.frames.main_frame.document.location.href='%s/'+this.options[this.selectedIndex].value;this.selectedIndex"0"><option value="%s/main_frame" disabled>New actions</option>""" %\
			(self.REQUEST['URL1'], self.relative_url())

		for action in self.get_workspace_create_actions():

			navigation_html = navigation_html + """<option value="%(action)s">%(title)s</option>""" % action

		return navigation_html + "</select>"


	def manage_afterAdd(self, item, container):
		"""We need to put certain functions here to avoid ZCatalog choking."""

		from Products.ZCatalog.ZCatalog import ZCatalog
		self._setObject('Catalog', ZCatalog('Catalog', title='', container=self))
		self['Catalog'].manage_addIndex('message_container_id', 'FieldIndex')
		self['Catalog'].manage_addIndex('value', 'TextIndex')
		self['Catalog'].manage_addIndex('sender', 'FieldIndex')
		self['Catalog'].manage_addIndex('body', 'TextIndex')
		self['Catalog'].manage_addIndex('read', 'FieldIndex')
		self['Catalog'].manage_addIndex('data', 'TextIndex')
		self['Catalog'].manage_addIndex('content_type', 'TextIndex')
		self['Catalog'].manage_addIndex('filename', 'TextIndex')
		self['Catalog'].manage_addIndex('date_created', 'FieldIndex')
		self['Catalog'].manage_addIndex('type', 'FieldIndex')
		self['Catalog'].manage_addIndex('relative_url', 'FieldIndex')

		self['Catalog'].manage_addColumn('read')
		self['Catalog'].manage_addColumn('sender')
		self['Catalog'].manage_addColumn('render_date')
		self['Catalog'].manage_addColumn('render_priority')
		self['Catalog'].manage_addColumn('render_size')
		self['Catalog'].manage_addColumn('render_html_subject')
		self['Catalog'].manage_addColumn('relative_url')

		util.install_workspace(self, username='morten', email='morten@esol.no')


	def get_workspace(self):
		"""Returns self."""

		return self


	def get_preferences_url(self):
		"""Returns the preferences url."""

		# FIXME -- we should break this up so
		# only 'preferences' is needed

		return self.relative_url() + '/preferences'


	def get_gum(self):
		"""Returns the mail client."""

		try:

			return self.objectValues('GUM')[0]

		except IndexError:

			return None


	def get_address_book(self):
		"""Returns the address book."""

		try:

			return self.objectValues('AddressBook')[0]

		except IndexError:

			return None


	def get_note_folder(self):
		"""Returns the notes."""

		try:

			return self.objectValues('Notes')[0]

		except IndexError:

			return None

	get_notes = get_note_folder


	def get_bookmark_folder(self):
		"""Returns the bookmark folder."""

		try:

			return self.objectValues('BookmarkFolder')[0]

		except IndexError:

			return None


	def get_calendar(self):
		"""Returns the calendar."""

		try:

			return self.objectValues('CalendarFolder')[0]

		except IndexError:

			return None


	get_calendar_folder = get_calendar


	def get_personal_folder(self):
		"""Returns the personal folder."""

		try:

			return self.objectValues('PersonalFolder')[0]

		except IndexError:

			return None


	def get_todo_folder(self):
		"""Returns the todo folder."""

		try:

			return self.objectValues('TodoFolder')[0]

		except IndexError:

			return None


	def get_news_host(self):
		"""Returns the newshost."""

		try:

			return self.objectValues('NewsHost')[0]

		except IndexError:

			return None


	def get_workspace_actions(self):
		"""Returns a list of workspace related product instance jumps."""

		actions = []

		gum = self.get_gum()

		if gum:

			actions.append(
				{
				'action':gum.relative_url(),
				'title':'Messenger'
				})

			#actions.append(
			#	{
			#	'action':gum.relative_url()+'/retrieve_messages',
			#	'title':'Get new'
			#	})

		calendar = self.get_calendar()

		if calendar:

			actions.append(
				{
				'action':calendar.get_current_day().relative_url(),
				'title':'Calendar'
				})

		address_book = self.get_address_book()

		if address_book:

			actions.append(
				{
				'action':address_book.relative_url(),
				'title':'Contacts'
				})

		notes = self.get_note_folder()

		if notes:

			actions.append(
				{
				'action':notes.relative_url(),
				'title':'Notes'
				})

		bookmarks = self.get_bookmark_folder()

		if bookmarks:

			actions.append(
				{
				'action':bookmarks.relative_url(),
				'title':'B\'marks'
				})

		personal_folder = self.get_personal_folder()

		if personal_folder:

			actions.append(
				{
				'action':personal_folder.relative_url(),
				'title':'Files'
				})

		if hasattr(self, 'get_workspace_manager'):

			actions.extend(self.get_workspace_manager().get_workspace_actions())

		return actions


	def get_workspace_create_actions(self):
		"""Returns a list of 'jump-actions'."""

		action_marker = '' #'- - - - '

		actions = []

		gum = self.get_gum()

		if gum:

			actions.append(
				{
				'action':gum.relative_url() + '/compose_mail',
				'title':action_marker + 'New message'
				})

			#actions.append(
			#	{
			#	'action':gum.get_primary_incoming_folder().relative_url(),
			#	'title':action_marker + gum.get_primary_incoming_folder().title
			#	})

		calendar = self.get_calendar()

		'''
		if calendar:

			actions.append(
				{
				'action':calendar.relative_url() + '/calendar_folder_add_event',
				'title':action_marker + 'New event'
				})

			actions.append(
				{
				'action':calendar.relative_url() + '/calendar_folder_add_todo',
				'title':action_marker + 'New todo'
				})
		'''

		address_book = self.get_address_book()

		if address_book:

			actions.append(
				{
				'action':address_book.relative_url() + \
					'/contacts/manage_add_address_book_contact_form',
				'title':action_marker + 'New contact'
				})

		notes = self.get_note_folder()

		if notes:

			actions.append(
				{
				'action':notes.relative_url() + '/manage_add_note_form',
				'title':'New note'
				})

		bookmark_folder = self.get_bookmark_folder()

		if bookmark_folder:

			actions.append(
				{
				'action':bookmark_folder.relative_url() + '/manage_add_bookmark_form',
				'title':'New bookmark'
				})

		if hasattr(self, 'get_workspace_manager'):

			actions.extend(self.get_workspace_manager().get_workspace_create_actions())

		return actions


	def get_workspace_preferences_actions(self):
		"""Returns a list of 'jump-actions'."""

		action_marker = '' #'- - - - '

		actions = []

		gum = self.get_gum()

		if gum:

			actions.append(
				{
				'action':gum.get_preferences_url(),
				'title':'Mail preferences'
				})

		calendar = self.get_calendar()

		if calendar:

			actions.append(
				{
				'action':calendar.get_preferences_url(),
				'title':'Calendar preferences'
				})

		address_book = self.get_address_book()

		if address_book:

			actions.append(
				{
				'action':address_book.get_preferences_url(),
				'title':'Address book preferences'
				})

		news_host = self.get_news_host()

		if news_host:

			actions.append(
				{
				'action':news_host.get_preferences_url(),
				'title':'NewsHost preferences'
				})


		'''
		actions.append(
			{
				'action':self.get_preferences_url(),
				'title':'Workspace preferences'
			})
		'''

		if getattr(self.aq_base, 'product_base_manage_theme', None):

			actions.append(
				{
				'action':self.relative_url() + '/product_base_manage_theme',
				'title':'Theme preferences'
				})

		actions.append(
			{
			'action':self.relative_url() + '/about',
			'title':'About workspace'
			})
			


		"""
		if address_book:

			actions.append(
				{
				'action':address_book.get_preferences_url(),
				'title':action_marker + 'AddressBook preferences'
				})

		notes = self.get_note_folder()

		if notes:

			actions.append(
				{
				'action':notes.get_preferences_url(),
				'title':'Notes preferences'
				})
		"""

		return actions


	def compose_mail(self, subject='', body='',
			recipients=[],
			newsgroups=[],
			REQUEST=None,
			RESPONSE=None):
		"""Starts a new message."""

		gum = self.get_gum()

		return gum.compose_mail(subject=subject, body=body,
					recipients=recipients, newsgroups=newsgroups,
					REQUEST=REQUEST, RESPONSE=RESPONSE)


	def make_event(self, title='', body='', REQUEST=None, RESPONSE=None,
			bookmarks=[]):
		"""Redirects to a screen with prefilled values for adding an event."""

		calendar = self.get_calendar()

		return calendar.calendar_folder_add_event(self, REQUEST=REQUEST, title_=title, body_=body,
					AUTHENTICATED_USER=REQUEST['AUTHENTICATED_USER'],
					render_day_widget=calendar.render_day_widget,
					render_month_widget=calendar.render_month_widget,
					render_year_widget=calendar.render_year_widget,
					relative_url=calendar.relative_url(),
					bookmarks=bookmarks,
					URL1=REQUEST['URL1'],
					RESPONSE=RESPONSE or REQUEST.RESPONSE)


	def make_todo(self, title='', body='', REQUEST=None, RESPONSE=None,
			bookmarks=[]):
		"""Redirects to a screen with prefilled values for adding a todo."""

		calendar = self.get_calendar()

		return calendar.calendar_folder_add_todo(self, REQUEST=REQUEST, title_=title, body_=body,
					AUTHENTICATED_USER=REQUEST['AUTHENTICATED_USER'],
					render_day_widget=calendar.render_day_widget,
					render_month_widget=calendar.render_month_widget,
					render_year_widget=calendar.render_year_widget,
					render_category_widget=calendar.render_category_widget,
					render_alternate_date_widget=calendar.render_alternate_date_widget,
					relative_url=calendar.relative_url(),
					bookmarks=bookmarks,
					URL1=REQUEST['URL1'],
					RESPONSE=RESPONSE or REQUEST.RESPONSE)


	def make_note(self, body='', REQUEST=None, RESPONSE=None):
		"""Redirects to a screen with prefilled values for adding a note."""

		notes = self.get_notes()

		return notes.manage_add_note_form(self, REQUEST=REQUEST, body_=body,
				AUTHENTICATED_USER=REQUEST['AUTHENTICATED_USER'],
				relative_url=notes.relative_url(),
				URL1=REQUEST['URL1'],
				RESPONSE=RESPONSE or REQUEST.RESPONSE)


	def add_event(self, title='', body='', priority=0,
			year=None, month=None, day=None,
			start=0, end=0, alarm=0,
			confirm_conflict_ignorance=None,
			RESPONSE=None):
		"""Adds an event in the calendar.

		This may raise a ConflictingEvents exception."""

		if None in (year, month, day):

			day = self.get_calendar().get_current_day()

		else:

			day = self.get_calendar().get_day_by_date(year, month, day)

		event = day.manage_add_calendar_event(title=title, note=body, alarm=alarm,
				start=start, end=end, priority=priority,
				confirm_conflict_ignorance=confirm_conflict_ignorance)

		if RESPONSE is not None:

			RESPONSE.redirect(event.relative_url())

		else:

			return event


	def add_todo(self, title='', body='', priority=0,
			year=None, month=None, day=None, 
			notify=None, RESPONSE=None):
		"""Adds a todo in the current day of the calendar."""

		calendar = self.get_calendar()

		if calendar is None: return

		if None in (year, month, day):

			new_todo = calendar.get_current_day().manage_add_todo(title, body, priority)

		else:

			new_todo = calendar.get_date(year, month ,day).manage_add_todo(title, body, priority)

		if notify:

			self.add_notification("Todo added")

		if RESPONSE is not None:

			RESPONSE.redirect(new_todo.relative_url())

		else:

			return new_todo


	def add_note(self, note, notify=None, RESPONSE=None):
		"""Adds a note in the notes folder in the home folder."""

		notes = self.get_note_folder()

		if notes is None: return

		note = notes.manage_add_note(body=note)

		if notify:

			self.add_notification("Note added")

		if RESPONSE is not None:

			RESPONSE.redirect(note.relative_url())

		else:

			return note


	def add_contact(self, firstname='', surname='', email='', notify=None, RESPONSE=None):
		"""Adds a contact in the address book in the home folder."""

		address_book = self.get_address_book()

		if address_book is None: return

		contact = address_book.manage_add_address_book_contact(firstname=firstname,
					surname=surname, email=email)

		if notify:

			self.add_notification("""Contact added (%s,%s,%s)""" % \
					((firstname + lastname) or email))

		if RESPONSE is not None:

			RESPONSE.redirect(contact.relative_url())

		else:

			return contact


	def check_triggers(self):
		"""A set of methods called to check new developments."""

		calendar_folder = self.get_calendar_folder()

		if calendar_folder:

			calendar_folder.get_current_day().check_alarms()
		

	def add_notification(self, message):
		"""Adds a notification to the notification queue."""

		try:

			self.getParentNode().add_notify_message(message)

		except AttributeError:

			self.notifications.append(message)


	def get_notifications(self, reset=None):
		"""Returns a list of notify messages."""

		if not self.use_notifications:

			return []

		notifications = self.notifications

		if reset:

			self.reset_notifications()

		return notifications


	def reset_notifications(self):
		"""Resets the notify list."""

		while 1:

			try:

				self.notifications = []
				get_transaction().commit()
				break

			except ZODB.POSException.ConflictError:

				time.sleep(0.1)


	def __init__(self, id, title='', description='',
			use_notifications=None,
			use_frames=None,
			install_default_html=None,
			install_default_theme=None,
			install_default_id_generator=None):
		"""Initialize the class instance."""

		self.id = id
		self.title = title
		self.description = description
		self.use_frames = use_frames
		self.use_notifications = use_notifications
		self.notifications = []

		product_base_item.utils.default_installation(self,
					install_default_html=install_default_html,
					install_default_theme=install_default_theme,
					install_default_id_generator=install_default_id_generator)


default__class_init__(workspace)

